/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.ProviderStatusInitializer");

dojo.require("dojox.lang.aspect");
dojo.require("jazz.app.context");

dojo.require("rmps.importer.Status");
dojo.require("rmps.importer.ImportClient");
dojo.require("dojox.html.entities")

dojo.require("jazz.ui.MessageBox");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "ProviderStatusInitializerMessages");

var messages = dojo.i18n.getLocalization("rmps.importer", "ProviderStatusInitializerMessages");

var States = rmps.importer.StatusStates;

// Calculates the cumulative state for the given statueses.
function calculateCumulativeState(providerStatuses) {
	var state = States.UNKNOWN;
	for (var i = 0, ps; ps = providerStatuses[i]; ++i) {
		var is = ps.importState;
		switch (state) {
			case States.UNKNOWN:
				if (is === States.UNKNOWN) {
					break;
				}
			case States.CANCELED:
				if (is === States.CANCELED) {
					state = States.CANCELED;
					break;
				}
			case States.COMPLETED:
				if (is === States.COMPLETED) {
					state = States.COMPLETED;
					break;
				}
			case States.WARNING:
				if (is === States.WARNING) {
					state = States.WARNING;
					break;
				}
			case States.IN_PROGRESS:
				if (is === States.IN_PROGRESS) {
					state = States.IN_PROGRESS;
					break;
				}
			case States.ERROR:
				if (is === States.ERROR) {
					state = States.ERROR;
					break;
				}
		}
	}
	return state;
}

/**
 * The Global Provider Status keeps track of the status of all
 * import content providers. Client can hook to the #onChange method
 * to receive updates. The provider status is updated every 30 seconds.
 * 
 * @author Christian Vogt (cvogt)
 */
rmps.importer.GlobalProviderStatus = {
	
	cumulativeState: rmps.importer.StatusStates.UNKNOWN,
	
	providerStatuses: [],
	
	onChange: function(self, changedProviderStatuses) {},
	
	start: function() {
		var context = jazz.app.context.get();
		var latch = true;
		if (context && context.itemId) {
			function get() {
				// This latch prevents multiple status requests from happening at once
				//  overloading an already heavily loaded server.
				if (!latch) {
					return;
				}
				latch = false;
				
				function restoreLatch() {
					latch=true;
				}
				
				var deferred = rmps.importer.ImportClient.getProviderStatus(context.itemId);
				deferred.addBoth(restoreLatch);
				deferred.addCallback(update);
			}
			
			function update(providerStatuses) {
				// calculate cumulative state
				var state = calculateCumulativeState(providerStatuses);
				var gps = rmps.importer.GlobalProviderStatus;
				
				// find only those statues that have chnaged state
				var changed = [];
				for (var i = 0, nps; nps = providerStatuses[i]; ++i) {
					var found = false;
					for (var j = 0, ps; ps = gps.providerStatuses[j]; ++j) {
						if (nps.providerName === ps.providerName) {
							found = true;
							if (nps.importState !== ps.importState) {
								//import status changed, need to reset cookie
								dojo.cookie("rmps.importer.GlobalProviderStatus",false,{expire:-1});
								changed.push(nps);
							}
							break;
						}
					}
					if (!found) {
						//page load, first check if user already has seen this message
						var hiddenStatus = dojo.cookie("rmps.importer.GlobalProviderStatus");
						if ( nps.importState.value !== hiddenStatus ) {
							dojo.cookie("rmps.importer.GlobalProviderStatus",false,{expire:-1});
							changed.push(nps);
						}
					}
				}
				
				var psCount = gps.providerStatuses.length;
				var psCS = gps.cumulativeState;
				gps.cumulativeState = state;
				gps.providerStatuses = providerStatuses;
				if (changed.length > 0 && (psCount !== 0 || state !== psCS)) {
					gps.onChange(gps, changed);
				}
			}
			
			get(); // update immediately
			setInterval(get, 30000); //update every 30 seconds
		}
	}
};

/**
 * Contributes the import provider status to the application nav bar.
 * 
 * @author Christian Vogt (cvogt)
 */
dojox.lang.aspect.advise(rmps.app.ui.internal.application.ModelingApplication, "start", {
	
	after: function() {
		var context = jazz.app.context.get();
		if (context && context.itemId) {
			var self = dojox.lang.aspect.getContext().instance;
			
			// We must sent a resize event when the message box is shown/hidden
			// and when the details are toggled.
			function onHideMessage() {
				//user closed message, set cookie to stop message from appearing at page reload
				dojo.cookie("rmps.importer.GlobalProviderStatus",messageBox.__cps[0].importState.value,{expires:90});
				delete messageBox.__cps;
				publishResizeEvent();
			}
			function publishResizeEvent() {
				dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
			}
			
			var messageBox = new jazz.ui.MessageBox();
			dojo.addClass(messageBox.domNode, "rmps-importer-provider-status-initializer");
			messageBox.domNode.style.margin = "1px";
			messageBox._first = true;
			dojo.connect(messageBox, "_toggleDetails", self, publishResizeEvent);
			dojo.connect(messageBox, "hideMessage", self, onHideMessage);
			dojo.connect(messageBox, "showMessage", self, publishResizeEvent);
			
			// Add the message box before the page container so that it will show up
			// no matter which page is currently being viewed.
			for (var i = 0, c; c = self.ui.containerNode.childNodes[i]; ++i) {
				if (dijit.byNode(c) instanceof net.jazz.ajax.ui.PageContainer) {
					self.ui.containerNode.insertBefore(messageBox.domNode, c);
					messageBox.startup();
					break;
				}
			}
			
			dojo.connect(rmps.importer.GlobalProviderStatus, "onChange", self, function(gps, cps) {
				if (cps.length > 0) {
					// copy the array
					cps = cps.slice();
					
					if (!messageBox.__cps) {
						messageBox.__cps = cps.slice();
					} else {
						// Find and replace old statues with their updated status.
						var changed = [];
						for (var i = 0, mbcps; mbcps = messageBox.__cps[i]; ++i) {
							var found = false;
							for (var j = 0, ps; ps = cps[j]; ++j) {
								if (mbcps.providerName === ps.providerName) {
									messageBox.__cps.splice(i, 1, ps);
									cps.splice(j, 1);
									break;
								}
							}
						}
						cps = messageBox.__cps = messageBox.__cps.concat(cps);
					}
					
					// Find the cumlative state in order to set the type
					// of the message box.
					var state = calculateCumulativeState(cps);
					if (messageBox._first) {
						delete messageBox._first;
						if (state === States.COMPLETED || state === States.UNKONWN) {
							return;
						}
					}
					
					var type;
					switch (state) {
						case States.COMPLETED:
							type = jazz.ui.MessageBox.STATUS_TYPES.OK;
							break;
							
						case States.WARNING:
							type = jazz.ui.MessageBox.STATUS_TYPES.WARNING
							break;
							
						case States.UNKNOWN:
						case States.IN_PROGRESS:
							type = jazz.ui.MessageBox.STATUS_TYPES.INFO;
							break;
							
						case States.CANCELED:
						case States.ERROR:
							type = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
							break;
					}
					
					if (cps.length === 1) {
						// If only 1 status, then show it without details.
						messageBox.showMessage({
							type: type,
							summary: dojo.string.substitute(messages.statusMessage, [
									dojox.html.entities.encode(messages[cps[0].importState.value + "Message"]),
									cps[0].importWebUri]),
							closable: true
						});
						
					} else {
						// Show the details for multiple statuses.
						var details = [];
						for (var i = 0, ps; ps = cps[i]; ++i) {
							details.push({message: '<img style="vertical-align: middle" src="' + ps.importState.icon + '"> ' + 
									dojo.string.substitute(messages.statusMessage, [
										dojox.html.entities.encode(ps.providerName),
										dojox.html.entities.encode(messages[ps.importState.value + "Message"]),
										ps.importWebUri])});
						}
						messageBox.showMessage({
							type: type,
							summary: dojo.string.substitute(messages.multiSummary, [cps.length]),
							closable: true,
							details: details
						});
					}
					
					// Replace the message box icon with the icon of the cumlative state.
					dojo.addClass(messageBox.messageArea, state.value);
				}
			});
			
			// Start the updating.
			rmps.importer.GlobalProviderStatus.start();
		}
	}
});

})();